      program roche
      implicit none
      include 'roche.dek'

! this program does the planer circular restricted three body problem (pcr3b). 
! the computation is done in a frame that is rotating with about the binary's center of mass,
! so that the binary is stationary. a nice reference is symon's mechanics.

! this code is hooked into mongo, both for edification purposes on how
! to do such things. one can plot the effective potential and watch an
! movie in an x11 window as the third body moves around.

! date code:  09jun1990 11feb2018

! declare

! general
      integer            :: i, j, k
      real*8             :: dum1, dum2


! for working between dimensional and dimensionless units
      integer            :: system_units
      real*8             :: m1_sav, m2_sav, d12_sav, omega_sav, &
                            mass_scale, distance_scale, time_scale, &
                            mass_scale_dimless, distance_scale_dimless, time_scale_dimless


! for the quintic to find the lagrange points
      external           :: quintic
      integer            :: iiter
      integer, parameter :: rdim=5
      real*8, parameter  :: tolroot = 1.0d-12
      real*8             :: ax(rdim), bx(rdim), lagx(rdim), lagy(rdim), zbrent, quintic


! for the effective potential 
      integer, parameter :: idim=200, jdim=200
      real*8             :: vdouble(idim,jdim), vxdouble(idim,jdim), vydouble(idim,jdim), &
                            xgrid(idim,jdim), ygrid(idim,jdim), &
                            vout, dvout_dx, dvout_dy, dvout_dt
      real*8, parameter  :: plot_scale = 1.5d0


! for the integration
      external           :: motion
      external           :: rkqs
      integer, parameter :: xdim = 1000000, ydim = 5, nbodies = 4
      real*8, parameter  :: tolode = 1.0d-8, odescal = 1.0d0, tiny=1.0d-16
      integer            :: nok, nbad, kount, ksav, iprint
      real*8             :: start, stopp, stptry, stpmin, tol2use, ode2use, xrk(xdim), yrk(ydim,xdim), bc(ydim)

! for interpolating the integrated solution
      integer            :: iat, nstep
      real*8             :: dt, time, period_one_orbit, yfer(4)


! for checking energy conservation
      real*8             :: energy_initial, energy_final, energy_change, energy, vsav


! constants
      real*8, parameter  :: g = 6.6742867d-8, &
                            msol = 1.9892d33, au = 1.495978921d13, sec_per_year = 3.1558149984d7, &
                            pi = 3.14159265358979323846264338327950d0


! declare; for the mongo plot stuff
      character*20       :: string
      character*80       :: long_string
      logical            :: xterm = .false., xpost = .false.
      character*20       :: psfile
      integer, parameter :: xpdim = 20000, ypdim = 5, nlev=10
      integer            :: icompute_potential, icompute_orbit, iplot_potential, iplot_orbit, ivector_field
      integer            :: jj, kk, imax, jmax, kmax, iterm, fileplot
      integer, parameter :: nstyle = 1
      real               :: vsingle(idim,jdim), vxsingle(idim*jdim), vysingle(idim*jdim), &
                            xsgrid(idim*jdim), ysgrid(idim*jdim), &
                            dxsgrid(idim*jdim), dysgrid(idim*jdim), &
                            vmin, vmax, vavr, zlev(nlev), alo, ahi, blo, bhi, style, &
                            xdum(xpdim), ydum(xpdim), xsav(ypdim,xpdim), ysav(ypdim,xpdim)
      real               :: colormap01(3,0:26)


! default x11 rgb color map
! easier and clearer than a transpose and shape initialization
      data    (colormap01(j,  0), j=1,3) / 0.0 , 0.0 , 0.0 /
      data    (colormap01(j,  1), j=1,3) / 1.0 , 1.0 , 1.0 /
      data    (colormap01(j,  2), j=1,3) / 0.5 , 0.5 , 0.5 /
      data    (colormap01(j,  3), j=1,3) / 0.5 , 0.0 , 0.0 /
      data    (colormap01(j,  4), j=1,3) / 1.0 , 0.0 , 0.0 /
      data    (colormap01(j,  5), j=1,3) / 1.0 , 0.0 , 0.5 /
      data    (colormap01(j,  6), j=1,3) / 0.5 , 0.0 , 0.5 /
      data    (colormap01(j,  7), j=1,3) / 1.0 , 0.0 , 1.0 /
      data    (colormap01(j,  8), j=1,3) / 1.0 , 0.5 , 1.0 /
      data    (colormap01(j,  9), j=1,3) / 1.0 , 0.5 , 0.5 /
      data    (colormap01(j, 10), j=1,3) / 1.0 , 0.5 , 0.0 /
      data    (colormap01(j, 11), j=1,3) / 1.0 , 1.0 , 0.0 /
      data    (colormap01(j, 12), j=1,3) / 1.0 , 1.0 , 0.5 /
      data    (colormap01(j, 13), j=1,3) / 0.5 , 0.5 , 0.0 /
      data    (colormap01(j, 14), j=1,3) / 0.5 , 1.0 , 0.0 /
      data    (colormap01(j, 15), j=1,3) / 0.0 , 0.5 , 0.0 /
      data    (colormap01(j, 16), j=1,3) / 0.0 , 1.0 , 0.0 /
      data    (colormap01(j, 17), j=1,3) / 0.5 , 1.0 , 0.5 /
      data    (colormap01(j, 18), j=1,3) / 0.0 , 1.0 , 0.5 /
      data    (colormap01(j, 19), j=1,3) / 0.0 , 0.5 , 0.5 /
      data    (colormap01(j, 20), j=1,3) / 0.0 , 1.0 , 1.0 /
      data    (colormap01(j, 21), j=1,3) / 0.5 , 1.0 , 1.0 /
      data    (colormap01(j, 22), j=1,3) / 0.0 , 0.5 , 1.0 /
      data    (colormap01(j, 23), j=1,3) / 0.0 , 0.0 , 0.5 /
      data    (colormap01(j, 24), j=1,3) / 0.0 , 0.0 , 1.0 /
      data    (colormap01(j, 25), j=1,3) / 0.5 , 0.5 , 1.0 /
      data    (colormap01(j, 26), j=1,3) / 0.5 , 0.0 , 1.0 /



! popular format statements
01    format(a80)
02    format(' ',12(' ',1pe10.3))


! set the types of actions to take
      icompute_potential = 1
      iplot_potential    = 1
      icompute_orbit     = 1
      iplot_orbit        = 1
      ivector_field      = 0



! calculation section 

! set the units 
! cgs
      mass_scale     = 1.0d0
      distance_scale = 1.0d0
      time_scale     = 1.0d0
      system_units   = 1

! solar masses, au, years
!      mass_scale     = msol
!      distance_scale = au
!      time_scale     = sec_per_year
!      system_units   = 2


! earth-moon system; m in earth masses, distances in earth-moon distances
!      mass_scale     = 5.972d27            ! earth masses
!      distance_scale = 3.844d10            ! average earth-moon distance
!      time_scale     = sec_per_year/12.0d0 ! approximate synodic month
!      system_units   = 3


! effective gravitational constant in the chosen units

      gprime  = g * mass_scale * (time_scale * time_scale) / (distance_scale*distance_scale*distance_scale)


! masses and seperation between the masses in the chosen units
      m1  = 1.0d0  
      m2  = 25.0d0 
      d12 = 10.0d0 

! earth-moon
!      m1  = 0.0123031469d0
!      m2  = 1.0d0
!      d12 = 1.0d0


! enforce m2 > m1
      if (m1 .gt. m2) then
       dum1 = m1
       m1   = m2
       m2   = dum1
      end if

! angular velocity of the binary, this sets the timescale

      omega = sqrt( ( (m1 + m2) * gprime ) / (d12*d12*d12) )  



! convert to dimensionless units for the calculation

! save the dimensional values
      m1_sav    = m1 
      m2_sav    = m2 
      d12_sav   = d12
      omega_sav = omega

      mass_scale_dimless     = 1.0d0/(m1 + m2)
      distance_scale_dimless = 1.0d0/d12
      time_scale_dimless     = 1.0d0/omega

! rescale user input
      m1     = m1 * mass_scale_dimless
      m2     = 1.0d0 - m1
      d12    = d12 * distance_scale_dimless
      omega  = omega * time_scale_dimless
      gprime = 1.0d0




! origin is at the center of mass
! positions xpos_mass1 of mass m1 and position xpos_mass2 of mass m2 on the x axis
      xpos_mass1   = ( m2 * d12 ) / (m1 + m2)
      xpos_mass2   = -1.0d0 * ( m1 * d12 ) / (m1 + m2)


! dimensionless positions zeta1 and zeta2
      zeta1 = xpos_mass1/d12
      zeta2 = zeta1 - 1.0d0



! larange point section
! the off-axis lagrange points.
! stable if m2/m1 > 0.5*(25 + 3 sqrt(69)) = 24.96, or m2 has 96% of the total mass.
! stable because the distances to the two masses are equal. the gravitational forces 
! from m1 and m2 are in the same ratio as the masses.

! l4
! note lagx = d12/2 * (m2-m1)/(m1+m2)

      lagx(4) = (zeta1 - 0.5d0) * d12
      lagy(4) = sqrt(3.0d0/4.0d0) * d12

! l5
      lagx(5) = lagx(4)
      lagy(5) = -1.0d0 * lagy(4)


! bracket all five of the on-axis roots in terms of the dimensionless distance zeta.
! from left (most negative) to right (most positive) the five roots are 
! l3, m2, l1, m1, l2 in this order.

      dum1 = -2.0d0
      dum2 =  2.0d0
      i = 200
      j = rdim
      call zbrak(quintic,dum1,dum2,i,ax,bx,j)
!      write(6,'(1x,i4,1p2e12.4)') (i,ax(i),bx(i), i=1,j)
      if (j.ne.rdim) then
       write(6,*) 'only bracketed ',j
       write(6,'(1x,i4,1p2e12.4)') (i,ax(i),bx(i), i=1,j)
       stop 'did not bracket all five roots'
      end if


! now find the on-axis, meta-stable lagrange points
! where the gravitational forces of m1 and m2 balance the centrifugal effect

! l3 - where the gravitational force of m1 and m2 causes m3 to orbit with the same period as m1.
! of m2 >> m1, then teh distance r from m2 to l3 is r ~ d12 * 7/12 * m1/m2 

      dum1 = zbrent(quintic,ax(1),bx(1),tolroot,iiter)
!      write(6,'(1x,1p10e12.4)') dum1,ax(1),bx(1)
      lagx(3) = d12 * dum1
      lagy(3) = 0.0d0


! l1 - intuitively where the gravitional forces of m1 and m2 balance.
! note is m2 >> m1, then the distance r from the m1 to l1 is equal
! to the hill sphere radius l1 ~ r ~ d12 * cube_root(m1/(3*M2))

      dum1 = zbrent(quintic,ax(3),bx(3),tolroot,iiter)
!      write(6,'(1x,1p10e12.4)') dum1,ax(3),bx(3)
      lagx(1) = d12 * dum1
      lagy(1) = 0.0d0


! l2 - 
! note is m2 >> m1, then the distance r from the m1 to l1 is equal
! to the hill sphere radius l1 ~ r ~ d12 * cube_root(m1/(3*M2))

      dum1 = zbrent(quintic,ax(5),bx(5),tolroot,iiter)
!      write(6,'(1x,1p10e12.4)') dum1,ax(5),bx(5)
      lagx(2) = d12 * dum1
      lagy(2) = 0.0d0

!      write(6,'(1x,i4,1p2e12.4)') (i,lagx(i),lagy(i), i=1,5)



! compute the effective potential on a grid
      if (icompute_potential .eq. 1) then
       write(6,*) ' computing the effective potential '
       call potential_grid(idim,jdim,vdouble,vxdouble,vydouble,xgrid,ygrid,plot_scale*d12)
       vmin = minval(vdouble)
       vmax = maxval(vdouble)
       vavr  = sum(vdouble)/float(idim*jdim)
!       write(6,*) vmin, vmax, vavr
      end if



! integrate the third body's equations of motion
      if (icompute_orbit .eq. 1) then
       write(6,*) ' integrating the equations of motion'

       start = 0.0d0 
       stpmin = 1.0e-20
       iprint = 0
       ksav   = 10000000

! for every body
       do k=1,nbodies

! initial conditions
! bc(1) is vx0 ; bc(2) is x0 ; bc(3) is vy0 ; bc(4) is y0 

        if (k .eq. 1) then

! circular kepler orbit about m2 
         bc(1) = 0.0d0
         dum1  = 0.3d0*d12
         bc(2) = xpos_mass2 - dum1
         bc(3) = 1.05d0 * (sqrt(m2 * gprime/abs(bc(2))) + omega*dum1)
         bc(4) = 0.0d0
         tol2use = tolode
         ode2use = odescal

! nice elliptical orbit in earth-moon system
!         bc(1) = 0.0d0
!         bc(2) = -0.75d0
!         bc(3) = 1.0d0
!         bc(4) = 0.0d0

! transfer orbit in earth-moon system
!         bc(1) = 0.04d0
!         bc(2) = 0.83d0
!         bc(3) = 0.0d0
!         bc(4) = 0.0d0


! set the stopping and storing times, and the initial step size to try
         period_one_orbit = 2.0d0 * pi * dum1 / bc(3) 
         stopp            = period_one_orbit * 100.0d0
         stptry           = period_one_orbit * 1.0d-12

! at the l4 lagrange point
        else if (k .eq. 2) then
         bc(1) = 0.0d0
         bc(2) = lagx(4)
         bc(3) = 0.0d0
         bc(4) = lagy(4)
         tol2use = tolode 
         ode2use = odescal

! offset from the l3 lagrange point
        else if (k .eq. 3) then
         bc(1) = 0.0d0
         bc(2) = lagx(3) * 1.0000001d0
         bc(3) = 0.0d0
         bc(4) = lagy(3) * 1.0000001d0
         tol2use = tolode 
         ode2use = odescal * 1.0d-3

! near the l5 lagrange point with a little shove to put it in orbit around l5
        else if (k .eq. 4) then
         bc(1) = -0.0002d0           
         bc(2) = lagx(5) - 0.0005d0 
         bc(3) = 0.0001d0            
         bc(4) = lagy(5) + 0.0005d0 
         tol2use = tolode
         ode2use = odescal * 1.0d-6
        end if


! the energy of mass of m3 is not a conserved quantity because the other two masses in the system are moving.
! initial energy

        call potential(bc(2),bc(4),bc(1),bc(3),vout,dvout_dx,dvout_dy,dvout_dt)

        if (k .eq. 1) vsav  = vout
        energy_initial = 0.5d0 * sqrt(bc(1)*bc(1) + bc(3)*bc(3)) + vout 
        bc(5) = energy_initial
        dum1  = bc(5)


! integrate
        call odeint(start,stptry,stpmin,stopp,bc, &
                    tol2use,xdim, &
                    xrk,yrk,xdim,ydim,xdim,ydim, &
                    nok,nbad,kount,ode2use,iprint, &
                    motion,rkqs)
        write(6,*) ' nok ',nok,' nbad ',nbad,' kount ',kount


! final energy
        call potential(bc(2),bc(4),bc(1),bc(3),vout,dvout_dx,dvout_dy,dvout_dt)
        energy_final = 0.5d0 * sqrt(bc(1)*bc(1) + bc(3)*bc(3)) + vout 

        energy_change = (energy_final - energy_initial) / (energy_initial + tiny)
        dum2 = (bc(5) - dum1)/bc(5)
        write(6,'(1x,1p10e14.6)') xrk(kount), energy_initial, energy_final, energy_change, dum1, bc(5), dum2


! interpolate the solution for plotting (only)
        iat   = 1
        dt = period_one_orbit/20.0
        nstep = int((stopp - start)/dt) + 1
        write(6,'(a,i5,a)') 'plotting ',nstep,' points'
        if (nstep .gt. xpdim) then
         write(6,*) 'nstep =',nstep
         write(6,*) 'xpdim  =',xpdim
         stop 'nstep > xdim'
        end if
        do i=1,nstep
         time = start + float(i-1)*dt
         call hunt(xrk,kount,time,iat)

         if (iat .eq. kount) exit

         yfer(1) = yrk(2,iat) 
         yfer(2) = yrk(2,iat+1)
         yfer(3) = yrk(2,iat+2)
         call hardwire_quadratic(xrk(iat),yfer,2,time,dum1)
         xsav(k,i) = dum1

         yfer(1) = yrk(4,iat) 
         yfer(2) = yrk(4,iat+1)
         yfer(3) = yrk(4,iat+2)
         call hardwire_quadratic(xrk(iat),yfer,kount,time,dum1)
         ysav(k,i) = dum1

!         write(6,'(2i4,1p14e12.4)') i,iat,time,xrk(iat),xrk(iat+1),yrk(2,iat),yrk(2,iat+1),yrk(4,iat),yrk(4,iat+1),xsav(k,i),ysav(k,i)
!         read(5,*)
        enddo
       
! end of body loop
       enddo

! end of icompute_orbit
      end if

444   continue


! ------------------------------

! plotting section 

! set kount to the number of stored values

      kount = nstep


! set the device, 11 is xterm 99 is postscript
      iterm = 11 
!      iterm = 99
      if (iterm .eq. 11) then
       xterm = .true.
      else if (iterm .eq. 99) then
       xpost = .true.
      else
       stop 'bad iterm value'
      end if

! set up the terminal; allow time on x terminals to expand the window
      if (xterm) then 
       call device(iterm)
       call tsetup

       call tidle
       write(6,*) 'resize window. <cr> when done'
       read(5,*)

      else if (xpost) then

       call setpsfmode(2)
       psfile = 'roche.ps'
       call psplot(.true.,psfile,' ')
!        call epsplot(.true.,psfile,' ')
       call margins(0.5, 0.5, 0.5, 0.5)
      end if


! set the colors after the device is initialized.
! flip black and white for postscript
       if (xpost) then 
        colormap01(1,0) = 1.0 ; colormap01(2,0) = 1.0 ; colormap01(3,0) = 1.0
        colormap01(1,1) = 0.0 ; colormap01(2,1) = 0.0 ; colormap01(3,1) = 0.0
       end if

       do i=0,26
        call makecolor(i,colormap01(1,i),colormap01(2,i),colormap01(3,i))
       enddo
       call setcolor(0)
       if (xpost) call setcolor(1)



      if (iplot_potential .eq. 1) call erase
      alo  = -plot_scale * d12
      ahi  = -alo
      blo  =  alo
      bhi  =  ahi
      call setlim(alo,blo,ahi,bhi)
      call setexpand(1.0)



! plot the equipotentials
      if (iplot_potential .eq. 1) then

! labels and axes
       call square(1,2,-1,-1)

! compose the title string
       write(string,'(a,0pf4.1)') 'M\d1=',m1/mass_scale_dimless
       call sqeeze(string)
       if (system_units .eq. 1) then
        string = trim(string) // ' g'
       else if (system_units .eq. 2) then
        string = trim(string) // ' Msun'
       else if (system_units .eq. 3) then
        string = trim(string) // ' M\dE'
       end if
       long_string = trim(string)

       write(string,'(a,0pf4.1)') 'M\d2=',m2/mass_scale_dimless
       call sqeeze(string)
       if (system_units .eq. 1) then
        string = trim(string) // ' g'
       else if (system_units .eq. 2) then
        string = trim(string) // ' Msun'
       else if (system_units .eq. 3) then
        string = trim(string) // ' M\dE'
       end if
       if (xterm) then
        long_string = trim(long_string) // '  ' // trim(string)
       else if (xpost) then
        long_string = trim(long_string) // '    ' // trim(string)
       end if

       write(string,'(a,0pf4.1)') 'd=',d12/distance_scale_dimless
       call sqeeze(string)
       if (system_units .eq. 1) then
        string = trim(string) // ' cm'
       else if (system_units .eq. 2) then
        string = trim(string) // ' AU'
       else if (system_units .eq. 3) then
        string = trim(string) // ' R\dE\dM'
       end if
       if (xterm) then
        long_string = trim(long_string) // '  ' // trim(string)
       else if (xpost) then
        long_string = trim(long_string) // '    ' // trim(string)
       end if

        write(string,'(a,1pe9.1)') 'P=',2.0d0*pi/(omega/time_scale_dimless)
        string(8:8) = 'e'
       call sqeeze(string)
       if (system_units .eq. 1) then
        string = trim(string) // ' s'
       else if (system_units .eq. 2) then
        string = trim(string) // ' yr'
       else if (system_units .eq. 3) then
        string = trim(string) // ' month'
       end if
       if (xterm) then
        long_string = trim(long_string) // '  ' // trim(string)
       else if (xpost) then
        long_string = trim(long_string) // '    ' // trim(string)
       end if

! write the title and axes labels
       call setexpand(1.0)
       if (xpost) call setexpand(1.0)
       call tlabel(len_trim(long_string),long_string)
       call setexpand(1.0)
       string = 'x (dimensionless)'
       call xlabel(len_trim(string),string)
       string = 'y (dimensionless)'
       call ylabel(len_trim(string),string)

       call setcolor(1)
       call setlweight(1.0)
       call setexpand(1.0)


! vector field
       if (ivector_field .eq. 1) then
        iiter = 4
        k = 0
        do i=1,idim,iiter
         do j=1,jdim,iiter
          if ( (sqrt( (xgrid(i,j) - xpos_mass1)**2 + ygrid(i,j)**2) .gt. 0.1) .and. &
               (sqrt((xgrid(i,j) - xpos_mass2)**2 + ygrid(i,j)**2) .gt. 0.2) ) then
           k = k + 1
           xsgrid(k)   = xgrid(i,j)
           ysgrid(k)   = ygrid(i,j)
           vxsingle(k) = -vxdouble(i,j)
           vysingle(k) = -vydouble(i,j)
          end if
         enddo
        enddo
        do i=1,k
         alo = 1.0d0/sqrt(vxsingle(i)*vxsingle(i) + vysingle(i)*vysingle(i))
         dxsgrid(i) = 4.0d0/3.0d0 * plot_scale/float(idim/iiter) * vxsingle(i)*alo
         dysgrid(i) = 4.0d0/3.0d0 * plot_scale/float(idim/iiter) * vysingle(i)*alo
        end do
        call setcolor(2)
        call setlweight(0.5)
        call vfield(xsgrid,ysgrid,dxsgrid,dysgrid,k,.false.)
        call setlweight(1.0)
       end if



! contour levels; artful choices
       call potential(lagx(1),lagy(1),dum1,dum2,vout,dvout_dx,dvout_dy,dvout_dt)
       zlev(1) = vout
       call potential(lagx(2),lagy(2),dum1,dum2,vout,dvout_dx,dvout_dy,dvout_dt)
       zlev(2) = vout
       call potential(lagx(3),lagy(3),dum1,dum2,vout,dvout_dx,dvout_dy,dvout_dt)
       zlev(3) = vout
       call potential(lagx(4),lagy(4),dum1,dum2,vout,dvout_dx,dvout_dy,dvout_dt)
       zlev(4) = vout
       call potential(lagx(5),lagy(5),dum1,dum2,vout,dvout_dx,dvout_dy,dvout_dt)
       zlev(5) = vout

       zlev(2) = zlev(2) * 0.98
       zlev(3) = zlev(3) * 1.00001
       zlev(4) = zlev(4) * 1.002
       zlev(5) = zlev(3) * 0.975
       zlev(6) = vsav
!       write(6,'(1x,i4,1pe10.2)') (i, zlev(i), i=1,6)

! draw the contours
       vsingle(1:idim,1:jdim) = vdouble(1:idim,1:jdim)
       call setlweight(2.0)
       if (xpost) call setlweight(1.0)
       call setcolor(22)
       call contour(vsingle,idim,jdim,zlev(1),1)
       call setcolor(22)
       call contour(vsingle,idim,jdim,zlev(2),1)
       call setcolor(22)
       call contour(vsingle,idim,jdim,zlev(3),1)
       call setcolor(26)
       call contour(vsingle,idim,jdim,zlev(4),1)
       call setcolor(26)
       call contour(vsingle,idim,jdim,zlev(5),1)
       call setlweight(1.0)
       call setcolor(2)
       call contour(vsingle,idim,jdim,zlev(6),1)
      end if


! draw the location of the mass points
      call setcolor(11)
      if (xpost) call setcolor(26)
      call setlvis(0)
      xdum(1) = xpos_mass1
      ydum(1) = 0.0
      xdum(2) = xpos_mass2
      ydum(2) = 0.0
      style = 203.0
      call setexpand(1.0)
      call points(style,nstyle,xdum(1),ydum(1),1,1)
      call setexpand(2.0)
      call points(style,nstyle,xdum(2),ydum(2),1,1)
      call setexpand(1.0)


! draw the lagrange points
      call setcolor(16)
      if (xpost) call setcolor(15)
      xdum(1:5) = lagx(1:5)
      ydum(1:5) = lagy(1:5)
      call setexpand(1.00)
      style = 203.0
      call points(style,nstyle,xdum,ydum,5,1)

! label lagrange points
      call setexpand(1.5)
      if (xpost) call setexpand(1.1)
      alo = xdum(1) - 0.15 * d12
      call relocate(alo,ydum(1))
      call putlabel(2,'L1',5)

      alo = xdum(2) + 0.25 * d12
      call relocate(alo,ydum(2))
      call putlabel(2,'L2',5)

      alo = xdum(3) - 0.15 * d12
      call relocate(alo,ydum(3))
      call putlabel(2,'L3',5)

      alo = xdum(4) - 0.08 * d12
      blo = ydum(4) + 0.15 * d12
      call relocate(alo,blo)
      call putlabel(2,'L4',5)

      alo = xdum(5) - 0.08 * d12
      blo = ydum(5) - 0.16 * d12
      call relocate(alo,blo)
      call putlabel(2,'L5',5)
      call setexpand(1.0)


! cross hairs at m2
!       call setcolor(26)
!       alo = 0.3 * d12
!       blo = zeta2
!       call relocate(blo,0.0)
!       call draw(-alo + blo,0.0)
!       call relocate(blo,0.0)
!       call draw(blo,alo)
!       call relocate(blo,0.0)
!       call draw(alo + blo,0.0)
!       call relocate(blo,0.0)
!       call draw(blo, -alo)

! crosshairs at center of mass
       call setcolor(1)
       alo = 0.03 * d12
       blo = 0.0
       call relocate(blo,0.0)
       call draw(-alo + blo,0.0)
       call relocate(blo,0.0)
       call draw(blo,alo)
       call relocate(blo,0.0)
       call draw(alo + blo,0.0)
       call relocate(blo,0.0)
       call draw(blo, -alo)
       call setlweight(1.0)
       call setcolor(1)



! plot the orbits
      if (xterm) then 
       call tidle
       write(6,*) 'start screen recording. <cr> when done'
       read(5,*)
      end if

      if (iplot_orbit .eq. 1) then
       call setcolor(4)
       call setlvis(2)
       call setexpand(1.5)
!       call setexpand(0.5)
       if (xpost) call setexpand(0.3)
       style = 203.0
       do i=1,kount
        if (mod(i,1000) .eq. 0) write(6,*) i
        do j=1,ydim
         xdum(j) = xsav(j,i)
         ydum(j) = ysav(j,i)
        enddo
        if (xterm) then
!         call setcolor(1)
!         alo = 2.0d0 * 0.9 * d12
!         call relocate(alo,alo)
!         write(string,'(a,1pe10.2)') 't=',xrk(i)
!         call setexpand(0.8)
!         call putlabel(len_trim(string),string,5)
!         call setexpand(1.0)

         call setcolor(4)
         call points(style,nstyle,xdum(1),ydum(1),1,1)
         call setcolor(7)
         call points(style,nstyle,xdum(2),ydum(2),1,1)
         call setcolor(10)
         call points(style,nstyle,xdum(3),ydum(3),1,1)
         call setcolor(2)
         call points(style,nstyle,xdum(4),ydum(4),1,1)
         call tidle

! sleepqq is likely to be compiler dependent
! the argument to sleepqq is the integer number of milliseconds to pause
! this contolls the speed, frames/sec, of the movie
         call sleepqq(40)
!          call sleepqq(1000)


! uncommentto get orbit trails
         if (i .ne. kount) then
!         call setcolor(1)
!         alo = 2.0d0 * 0.9 * d12
!         call relocate(alo,alo)
!         write(string,'(a,1pe10.2)') 't=',xrk(i)
!         call setexpand(0.8)
!         call putlabel(len_trim(string),string,5)
!         call setexpand(1.0)

          call setcolor(4)
          call points(style,nstyle,xdum(1),ydum(1),1,1)
          call setcolor(7)
          call points(style,nstyle,xdum(2),ydum(2),1,1)
          call setcolor(10)
          call points(style,nstyle,xdum(3),ydum(3),1,1)
          call setcolor(2)
          call points(style,nstyle,xdum(4),ydum(4),1,1)
          call tidle
         end if


        else if (xpost) then
!         call setcolor(4)
!         call points(style,nstyle,xdum(1),ydum(1),1,1)
!         call setcolor(7)
!         call points(style,nstyle,xdum(2),ydum(2),1,1)
!         call setcolor(10)
!         call points(style,nstyle,xdum(3),ydum(3),1,1)
!         call setcolor(2)
!         call points(style,nstyle,xdum(4),ydum(4),1,1)
        end if

! end of evolution loop for xterm
       enddo
       

if (xpost) then
        call setcolor(4)
        k = kount
        xdum(1:k) = xsav(1,1:k)
        ydum(1:k) = ysav(1,1:k)
        call connect(xdum,ydum,k)

        call setcolor(7)
        xdum(1:kount) = xsav(2,1:kount)
        ydum(1:kount) = ysav(2,1:kount)
        call connect(xdum,ydum,kount)

        call setcolor(10)
        xdum(1:kount) = xsav(3,1:kount)
        ydum(1:kount) = ysav(3,1:kount)
        call connect(xdum,ydum,kount)

        call setcolor(2)
        xdum(1:kount) = xsav(4,1:kount)
        ydum(1:kount) = ysav(4,1:kount)
        call connect(xdum,ydum,kount)
       end if
      end if


! thats all
      if (xterm) then
       call tidle
       write(6,*) 'hit return <cr> to end'
       read(5,*)
      else if (xpost) then
       i = fileplot(0)
       write(6,*) i,' vectors plotted'
      end if

      stop 'normal termination'
      end program roche






      subroutine potential(xin,yin,vxin,vyin,vout,dvout_dx,dvout_dy,dvout_dt)
      implicit none
      include 'roche.dek'

! given the x and y coordinates, xin and yin, return vout, the effective potential energy
! of the restricted three body problem in the rotating frame. also returned are the
! first derivatives with respect to xin, dvout_dx, and with respect to yin, dvout_dy.
!
! given the velocity componentx vxin and vyin, also return the derivative with respect to time, dvout_dt.


! declare the pass
      real*8, intent(in)  :: xin, yin, vxin, vyin
      real*8, intent(out) :: vout, dvout_dx, dvout_dy, dvout_dt

! local variables
      real*8              :: mu1, f1, f12, f13, mu2, f2, f22, f23, om2


! common terms
      mu1 = m1 * gprime
      f1  = (xin-xpos_mass1)*(xin-xpos_mass1) + yin*yin
      f12 = sqrt(f1)
      f13 = f1 * f12

      mu2 = m2 * gprime
      f2  = (xin-xpos_mass2)*(xin-xpos_mass2) + yin*yin 
      f22 = sqrt(f2)
      f23 = f2 * f22

      om2 = omega*omega

! the effective potential energy per unit mass of the third body
! first two terms are gravitational, third term is centrifugal

      vout = mu1/f12 + mu2/f22  + 0.5d0*(xin*xin + yin*yin)*om2
      vout = -1.0d0 * vout


! partial with respect to x
      dvout_dx = mu1/f13*(xin-xpos_mass1) + mu2/f23*(xin-xpos_mass2) - xin*om2


! partial with respect to y
      dvout_dy = mu1/f13*yin + mu2/f23*yin - yin*om2


! partial with time
      dvout_dt =   mu1/f13*((xin-xpos_mass1)*vxin + yin*vyin)  &
                 + mu2/f23*((xin-xpos_mass2)*vxin + yin*vyin)  &
                 - (xin*vxin + yin*vyin)*om2

      return
      end subroutine potential





      subroutine motion(t,f,dfdt)
      implicit none
      save

      include 'roche.dek'

! this routine contains the system of first order odes representing the
! motion of the perturbing third body in the rotating frame of the
! restricted three body problem.

! notation is
! t    = time
! f(1) = vx, velocity in x direction
! f(2) =  x, position in x direction
! f(3) = vy, velocity in y direction
! f(4) =  y, position in y direction


! declare the pass
      real*8  :: t,f(4),dfdt(4)

! local variables
      real*8  :: vout, dvout_dx, dvout_dy, dvout_dt


! get the derivatives of the effectve potential 
      call potential(f(2),f(4),f(1),f(3),vout,dvout_dx,dvout_dy,dvout_dt)

! x-coordinate ; second term on right is the coriolis acceleration
      dfdt(1) = -dvout_dx + 2.0d0*omega*f(3)
      dfdt(2) = f(1)

! y-coordinate; ; second term on right is the coriolis acceleration
      dfdt(3) = -dvout_dy - 2.0d0*omega*f(1)
      dfdt(4) = f(3)

! d(energy)/dt = power. analytically this is zero.
      dfdt(5) = f(1)*dfdt(1) + f(3)*dfdt(3) + dvout_dt

      return
      end subroutine motion





      subroutine potential_grid(idim,jdim,v,vx,vy,xgrid,ygrid,range)
      implicit none

! this routine stores in memory and disk the effective potential 
! on a 2d grid for the restricted three body problem in the rotating frame.

! declare the pass
      integer, intent(in) :: idim,jdim
      real*8, intent(in)  :: range
      real*8, intent(out) :: v(idim,jdim), vx(idim,jdim), vy(idim,jdim), &
                             xgrid(idim,jdim), ygrid(idim,jdim)

! local variables
      integer :: i,j
      real*8  :: x, dx, y, dy, stp, xlo, xhi, ylo, yhi, dvout_dx, dvout_dy, dvout_dt


! open the data file
      open(unit=2,file='veff.dat', status='unknown')

! limits 
      xlo  = -range
      xhi  =  range
      ylo  = -range
      yhi  =  range
      stp = (xhi - xlo)/float(idim - 1)

! sweep through the 2d grid
      do i=1,idim
       x = xlo + float(i-1) * stp
       do j=1,jdim
        y = ylo + float(j-1) * stp
        call potential(x,y,0.0d0,0.0d0,v(i,j),vx(i,j),vy(i,j),dvout_dt)
        write(2,10) i,j,v(i,j),x,y
10      format(1x,i4,i4,1p3e10.2)
        xgrid(i,j) = x
        ygrid(i,j) = y
       enddo
       write(2,*) ' '
      enddo

! close up shop
      close(unit=2)
      return
      end subroutine potential_grid




      real*8 function quintic(zeta)
      implicit none
      include 'roche.dek'

! this function represents the quintic equation in the dimensionless distance zeta
! for the on-axis lagrange points in the rotating frame.

! input:
! zeta  - dimensionless position
! zeta1 - dimensionaless position of mass m1 via commonblock
! zeta2 - dimensionaless position of mass m2 via commonblock
!
! output is quintic, the function whose zeros we seek
!
! along y=0, the x-component of the acceleration, the negative derivative of the potential with respect to x, is zero when
!
! zeta   + zeta2 * (zeta - zeta1) / |zeta - zeta1|^3   - zeta1 * (zeta - zeta2) / |zeta - zeta2|^3 = 0
!
! a quintic in zeta. 
! the absolute values arise from sqrt(x^2) = |x|, and note |x|^n = |x^n|.
! note we have "+ zeta2 *" instead of "- zeta2" because the definition of zeta2.
!
! to avoid dividing by zero at zeta=zeta1 and zeta=zeta2 this routine
! multiplies the above expression by |zeta - zeta1|  * |zeta - zeta2|.
! 
!
! from left (most negative) to right (most positive) the five roots
! are l3, m2, l1, m1, l2 in this order. 

! declare the pass
      real*8 :: zeta

! local varibles
      real*8 :: a,b,c,d,x

! go 
      a = zeta - zeta1
      x = abs(a) 
      c = x * x * x

      b = zeta - zeta2
      x = abs(b)
      d = x * x * x

      quintic = zeta*c*d + zeta2*a*d - zeta1*b*c
      return
      end function quintic




      subroutine odeint(start,stptry,stpmin,stopp,bc, &
                        eps, kmax, &
                        xrk,yrk,xphys,yphys,xlogi,ylogi, &
                        nok,nbad,kount,odescal,iprint, &
                        derivs,steper)
      implicit none
      save

! input are the initial value to start the integration from start, the first
! step size stptry, minimum step size stpmin, the ending value of the
! integration stopp, the initial conditions bc, the desired fractional
! accuracy eps, the maximum number of values to store kmax, the array xrk 
! (physical dimension xphys and logical dimension xlogi) where the solution was
! computed between start and stopp, the array yrk (physical dimension
! yphys,xphys and logical dimension ylogi,xlogi) where the solution to the
! ylogi ode's are stored, the error scaling factor odescal, a flag iprint
! to determine if we print the solution to the screen as it evolves, the
! name of the subroutine derivs that sets up the ode's and
! finally the name of the subroutine steper that will do the integrations
! (valid choices include rkqc, rkqs, bsstep and stiff).
!
! output are the number of succesful steps taken nok, the number of bad
! (but retried and then succesful) steps taken, the total number of steps
! taken kount and of course the solution arrays xrk and yrk described above.
!
! nmax is the maximum number of differential equations
!
! declare the pass
      external              :: derivs,steper
      integer, intent(in)   :: xphys, yphys, xlogi, ylogi, kmax, iprint
      integer, intent(out)  :: nok, nbad, kount
      real*8, intent(in)    :: start, stptry, stpmin, stopp, eps, odescal
      real*8, intent(inout) :: bc(yphys)
      real*8, intent(out)   :: xrk(xphys), yrk(yphys,xphys)


! local variables
      integer            :: i,j,nstp
      integer, parameter :: nmax = 10, stpmax=1000000
      real*8             :: yscal(nmax), y(nmax), dydx(nmax), x, xsav, h, hdid, hnext
      real*8, parameter  :: zero=0.0d0, one=1.0d0, tiny=1.0d-30, ttiny=1.0d-15


! popular format statements
100   format(1x,i4,1pe10.2)
101   format(1x,1p12e10.2)


! initialize
      if (ylogi .gt. yphys) stop 'ylogi > yphys in routine odeint'
      if (yphys .gt. nmax)  stop 'yphys > nmax in routine odeint'
      x     = start
      h     = sign(stptry,stopp-start)
      nok   = 0
      nbad  = 0
      kount = 0

! store the first step
      y(1:ylogi) = bc(1:ylogi)
      xsav = x  

! take at most stpmax steps
      do nstp=1,stpmax
       call derivs(x,y,dydx)

! scaling vector used to monitor accuracy
       do i=1,ylogi

! constant fractional accuracy
!        yscal(i) = odescal * abs(y(i)) + tiny

! const. frac. cept near zero
!         yscal(i)=abs(y(i)) + abs(h*dydx(i)) + tiny

! step size dependent accuracy
!          yscal(i) = abs(odescal * h * dydx(i)) + tiny

! for stiffs
!         yscal(i) = max(odescal,y(i))

! straight scaling (decrease to get more time steps)
         yscal(i) = odescal

       enddo

! store the intermediate results
       if (kmax .gt. 0) then
         if ( kount .lt. (kmax-1) ) then
          kount         = kount+1
          xrk(kount)    = x
          yrk(1:ylogi,kount) = y(1:ylogi)
          if (iprint .eq. 1) then
           write(6,100) kount,xrk(kount)
           write(6,101) (yrk(j,kount), j=1,ylogi)
          end if
          xsav = x
         end if
       end if


! if the step can overshoot the stop point then cut it
       if ((x+h-stopp)*(x+h-start) .gt. zero) h = stopp - x

! do an integration step
       call steper(y,dydx,ylogi,x,h,eps,yscal,hdid,hnext,derivs)
       if (hdid.eq.h) then
        nok = nok+1
       else
        nbad = nbad+1
       end if

! this is the normal exit point, save the final step
       if (nstp .eq. stpmax .or. (x-stopp)*(stopp-start) .ge. zero) then
        bc(1:ylogi) = y(1:ylogi)
        if (kmax.ne.0) then
         kount         = kount+1
         xrk(kount)    = x
         yrk(1:ylogi,kount) = y(1:ylogi)
         if (iprint .eq. 1) then
          write(6,100) kount,xrk(kount)
          write(6,101) (yrk(j,kount), j=1,ylogi)
         end if
        end if
        return
       end if

! set the step size for the next iteration; stay above stpmin
       h = hnext
       if (abs(hnext) .lt. stpmin) then
        write(6,'(1x,a,1pe14.6)') 'hnext = ',hnext
        write(6,'(1x,a,1pe14.6)') 'stpmin = ',stpmin
        stop 'hnext < stpmin in odeint'
       end if

! back for another iteration or death
      enddo
      write(6,*) '> than stpmax steps required in odeint'
      return
      end subroutine odeint



! explicit:
! routine rkqs is a rkck integrator; plug into odeint
! routine rkck is a fifth order cash-karp runge-kutta stepper



      subroutine rkqs(y,dydx,n,x,htry,eps,yscal,hdid,hnext,derivs)
      implicit none
      save
      
! fifth order cash-karp runge kutta step with monitoring of the local
! truncation error to ensure accuracy and adjust stepsize. input are the
! dependent variable vector y(1:n) and its derivative dydx(1:n) at the
! starting value of the independent variable x. also input are the stepsize
! to be attempted htry, the required accuracy eps, and the vector yscal(1:n)
! against which the error is scaled. on output, y and x are replaced with
! their new values, hdid is the stepsize that was actually accomplished, and
! hnext is the estimated next stepsize. derivs is a usr supplied function
! that supplies the right hand sides. plug into odeint.

! declare the pass
      external              :: derivs
      integer, intent(in)   :: n
      real*8, intent(in)    :: dydx(n), htry, eps, yscal(n)
      real*8, intent(inout) :: y(n), x
      real*8, intent(out)   :: hdid, hnext

! local variables
      integer            :: i
      integer, parameter :: nmax = 10
      real*8             :: errmax, h, xnew, yerr(nmax), ytemp(nmax),htemp
      real*8, parameter  :: safety = 0.9d0, pgrow = -0.2d0, pshrnk = -0.25d0, & 
                            errcon = (5.0d0/safety)**(1.0d0/pgrow)

! set stepsize to the initial trial routine
      h = htry
01    call rkck(y,dydx,n,x,h,ytemp,yerr,derivs)

! evaluate accuracy
      errmax = 0.0d0
      do i =1,n
       errmax = max(errmax,abs(yerr(i)/yscal(i)))
      enddo
      errmax = errmax/eps

! truncation error too large, reduce stepsize by a maximum of 10
      if (errmax .gt. 1.0) then
       htemp = safety * h * (errmax**pshrnk)
       h     = sign(max(abs(htemp),0.1d0*abs(h)),h)
       xnew = x + h
       if (xnew .eq. x) stop 'stepsize too small in rkqs'
       go to  01

! step succeeded. compute size of next step.
      else
       if (errmax .gt. errcon) then
        hnext = safety * h * (errmax**pgrow)
       else
        hnext = 5.0d0 * h
       end if
       hdid = h
       x    = x + h
       y(1:n) = ytemp(1:n)
      end if
      return
      end subroutine rkqs





      subroutine rkck(y,dydx,n,x,h,yout,yerr,derivs)
      implicit none
      save

! given value for the n variables y and their derivatives dydx known at x,
! use the fifth-order cash-karp runge-kutta method to advance the solution
! over an interval h, and return the incremented variables as yout. also
! return an estimate of the local truncation error in yout using the
! embedded fourth order method. the user supplies the routine derivs which
! computes the right hand side.

! declare the pass
      external            :: derivs
      integer,intent(in)  :: n
      real*8, intent(in)  :: y(n), dydx(n), x, h
      real*8, intent(out) :: yout(n), yerr(n)

! local variables
      integer            :: i
      integer, parameter :: nmax = 10
      real*8             ::  ak2(nmax),ak3(nmax),ak4(nmax),ak5(nmax),ak6(nmax),ytemp(nmax)

! cash and karp values
      real*8, parameter :: a2  = 0.2d0, a3 = 0.3d0, a4 = 0.6d0, a5 = 1.0d0, &
                           a6  = 0.875d0, b21 = 0.2d0, b31 = 3.0d0/40.0d0, &
                           b32 = 9.0d0/40.0d0, b41 = 0.3d0, b42 = -0.9d0, &
                           b43 = 1.2d0, b51 = -11.0d0/54.0d0, b52 = 2.5d0, &
                           b53 = -70.0d0/27.0d0, b54 = 35.0d0/27.0d0, &
                           b61 = 1631.0d0/55296.0d0, b62 = 175.0d0/512.0d0, &
                           b63 = 575.0d0/13824.0d0, &
                           b64 = 44275.0d0/110592.0d0, &
                           b65 = 253.0d0/4096.0d0, c1 = 37.0d0/378.0d0, &
                           c3  = 250.0d0/621.0d0, c4 = 125.0d0/594.0d0, &
                           c6  = 512.0d0/1771.0d0, &
                           dc1 = c1 - 2825.0d0/27648.0d0, &
                           dc3 = c3 - 18575.0d0/48384.0d0, &
                           dc4 = c4 - 13525.0d0/55296.0d0, &
                           dc5 = -277.0d0/14336.0d0, &
                           dc6 = c6 - 0.25d0

! first step
       do i=1,n
        ytemp(i) = y(i) + h*b21*dydx(i)
       enddo

! second step
       call derivs(x+a2*h,ytemp,ak2)
       do i=1,n
        ytemp(i) = y(i) + h*(b31*dydx(i) + b32*ak2(i))
       enddo

! third step
       call derivs(x+a3*h,ytemp,ak3)
       do i=1,n
        ytemp(i) = y(i) + h*(b41*dydx(i) + b42*ak2(i) + b43*ak3(i))
       enddo

! fourth step
       call derivs(x+a4*h,ytemp,ak4)
       do i=1,n
        ytemp(i) = y(i) + h*(b51*dydx(i) + b52*ak2(i) + b53*ak3(i) + b54*ak4(i))
       enddo

! fifth step
       call derivs(x+a5*h,ytemp,ak5)
       do i=1,n
        ytemp(i) = y(i) + h*(b61*dydx(i) + b62*ak2(i) + b63*ak3(i) + b64*ak4(i) + b65*ak5(i))
       enddo

! sixth step; accumulate increments with proper weights
       call derivs(x+a6*h,ytemp,ak6)
       do i=1,n
        yout(i) = y(i) + h*(c1*dydx(i) + c3*ak3(i) + c4*ak4(i) + c6*ak6(i))
       enddo

! estimate error as difference between fourth and fifth order methods
      do i=1,n
       yerr(i) = h*(dc1*dydx(i) + dc3*ak3(i) + dc4*ak4(i) + dc5*ak5(i) + dc6*ak6(i))
      enddo
      return
      end subroutine rkck





! root brackets and root finders

      subroutine zbrak(fx,x1,x2,n,xb1,xb2,nb)
      implicit none
      save 

! given a function fx defined on the interval x1 to x2, subdivide the
! interval into n equally spaced segments, and search for zero crossings
! of the function. nb is input as the maximum number of roots sought and
! is reset to the number of bracketing pairs xb1,xb2 that are found.

! declare the pass
      external               :: fx
      real*8                 :: fx
      integer, intent(in)    :: n
      integer, intent(inout) :: nb
      real*8, intent(in)     :: x1,x2
      real*8, intent(out)    :: xb1(nb),xb2(nb)


! local variables
      integer   :: nbb,i
      real*8    :: x,dx,fp,fc

! initialize and determine the appropriate mesh spacing
      nbb = 0
      x   = x1
      dx  = (x2-x1)/n
      fp  = fx(x)

! loop over all intervals
      do i=1,n
       x  = x + dx
       fc = fx(x)

! if a sign change occurs then record the values for the bounds
       if (fc*fp .le. 0.0d0) then
        nbb      = nbb + 1
        xb1(nbb) = x - dx
        xb2(nbb) = x
        if (nbb .eq. nb) goto 10
       end if
       fp = fc
      enddo
10    nb = nbb
      return
      end subroutine zbrak





      real*8 function zbrent(func,x1,x2,tol,niter)
      implicit none
      save

! using brent's method this routine finds the root of a function func
! between the limits x1 and x2. the root is when accuracy is less than tol.

! note: eps the the machine floating point precision

! declare the pass
      external             :: func
      real*8               :: func
      real*8, intent(in)   :: x1, x2, tol
      integer, intent(out) :: niter


! local variables
      integer            :: iter
      integer, parameter :: itmax = 100
      real*8             :: a,b,c,d,e,fa,fb,fc,xm,tol1,p,q,r,s
      real*8             :: eps=3.0d-15


! initialize
      niter = 0
      a     = x1
      b     = x2
      fa    = func(a)
      fb    = func(b)
      if ( (fa .gt. 0.0d0  .and. fb .gt. 0.0d0)  .or. &
           (fa .lt. 0.0d0  .and. fb .lt. 0.0d0)       ) then
       write(6,100) x1,fa,x2,fb
100    format(1x,' x1=',1pe11.3,' f(x1)=',1pe11.3,/, &
              1x,' x2=',1pe11.3,' f(x2)=',1pe11.3)
       stop 'root not bracketed in routine zbrent'
      end if
      c  = b
      fc = fb

! rename a,b,c and adjusting bound interval d
      do iter =1,itmax
       niter = niter + 1
       if ( (fb .gt. 0.0d0  .and. fc .gt. 0.0d0)  .or. &
            (fb .lt. 0.0d0  .and. fc .lt. 0.0d0)      ) then
        c  = a
        fc = fa
        d  = b-a
        e  = d
       end if
       if (abs(fc) .lt. abs(fb)) then
        a  = b
        b  = c
        c  = a
        fa = fb
        fb = fc
        fc = fa
       end if
       tol1 = 2.0d0 * eps * abs(b) + 0.5d0 * tol
       xm   = 0.5d0 * (c-b)

! convergence check
       if (abs(xm) .le. tol1 .or. fb .eq. 0.0d0) then
        zbrent = b
        return
       end if

! attempt quadratic interpolation
       if (abs(e) .ge. tol1 .and. abs(fa) .gt. abs(fb)) then
        s = fb/fa
        if (a .eq. c) then
         p = 2.0d0 * xm * s
         q = 1.0d0 - s
        else
         q = fa/fc
         r = fb/fc
         p = s * (2.0d0 * xm * q *(q-r) - (b-a)*(r - 1.0d0))
         q = (q - 1.0d0) * (r - 1.0d0) * (s - 1.0d0)
        end if

! check if in bounds
        if (p .gt. 0.0d0) q = -q
        p = abs(p)

! accept interpolation
        if (2.0d0*p .lt. min(3.0d0*xm*q - abs(tol1*q),abs(e*q))) then
         e = d
         d = p/q

! or bisect
        else
         d = xm
         e = d
        end if

! bounds decreasing to slowly use bisection
       else
        d = xm
        e = d
       end if

! move best guess to a
       a  = b
       fa = fb
       if (abs(d) .gt. tol1) then
        b = b + d
       else
        b = b + sign(tol1,xm)
       end if
       fb = func(b)
      enddo
      stop 'too many iterations in routine zbrent'
      end function zbrent






      subroutine sqeeze(line)
      implicit none

! this routine takes line and removes all blanks, such as
! those from writing to string with fortran format statements
!
! declare the pass
      character*(*)  line

! local variables
      character*1    achar
      integer        l,n,k,lend,lsiz


! find the end of the line
      lsiz = len(line)
      lend = len_trim(line)
      n    = 0
      l    = 0

! do the compression in place
10    continue
      l = l + 1
      achar = line(l:l)
      if (achar .eq. ' ') goto 10
      n = n + 1
      line(n:n) = achar
      if (l .lt. lend) goto 10

! blank the rest of the line
      do k=n+1,lsiz
       line(k:k) = ' '
      enddo
      return
      end subroutine sqeeze




! routine hunt finds a table entry given a guess for a nearby entry


      subroutine hunt(xx,n,x,jlo)
      implicit none
      save

! given an array xx of length n and a value x, this routine returns a value
! jlo such that x is between xx(jlo) and xx(jlo+1). the array xx must be
! monotonic. j=0 or j=n indicates that x is out of range. on input jlo is a
! guess for the table entry, and should be used as input for the next hunt.
! one assumes the next table entry is relatively close to the old one.

! declare
      logical          ascnd
      integer          n,jlo,jhi,inc,jm
      real*8 xx(n),x

! logical function; true if ascending table, false if descending
      ascnd = xx(n) .ge. xx(1)

! horrid initial guess; go to bisection immediatly
      if (jlo.le.0 .or. jlo.gt.n) then
       jlo = 0
       jhi = n+1
       goto 3
      end if

! set the initial hunting increment
      inc = 1

! this is the hunt up section
      if (x.ge.xx(jlo) .eqv. ascnd) then
1      jhi = jlo + inc

! if we are the end of the table then we are done hunting
       if (jhi .gt. n) then
        jhi = n + 1

! otherwise replace the lower limit, double the increment and try again
       else if (x.ge.xx(jhi) .eqv. ascnd) then
        jlo = jhi
        inc = inc + inc
        goto 1
       end if

! this is the hunt down section
      else
       jhi = jlo
2      jlo = jhi - inc

! if we are the end of the table then we are done hunting
       if (jlo.lt.1) then
        jlo = 0

! otherwise replace the upper limit, double the increment and try again
       else if (x.lt.xx(jlo) .eqv. ascnd) then
        jhi = jlo
        inc = inc + inc
        goto 2
       end if
      end if

! this is the final bisection phase
3     if (jhi-jlo .eq. 1) then
       if (x .eq. xx(n)) jlo = n - 1
       if (x .eq. xx(1)) jlo = 1
       return
      end if
      jm = (jhi+jlo)/2
      if (x .ge. xx(jm) .eqv. ascnd) then
       jlo = jm
      else
       jhi = jm
      end if
      goto 3
      end




! hardwired polynomial interpolation routines
! hardwire_linear does linear interpolation
! hardwire_quadratic does quadratic interpolation
! hardwire_cubic does cubic interpolation


      subroutine hardwire_linear(xtab,ytab,n,xin,yout)
      implicit none

! implements a hardwired linear interpolation

! declare the pass
      integer   :: n
      real*8    :: xtab(n),ytab(n),xin,yout

! local variables
      real*8    :: x1,x2,denom,alfa,beta

      x1    = xtab(1)
      x2    = xtab(2)
      denom = 1.0d0/(x1-x2)
      alfa  = (xin - x2)*denom
      beta  = (x1 - xin)*denom
      yout  = alfa*ytab(1) + beta*ytab(2)

      return
      end subroutine hardwire_linear





      subroutine hardwire_quadratic(xtab,ytab,n,xin,yout)
      implicit none

! implements a hardwired quadratic interpolation

! declare the pass
      integer  :: n
      real*8   :: xtab(n),ytab(n),xin,yout

! local variables
      real*8   :: x1,x2,x3,a,b,c,d,e,f,alfa,beta,gama

      x1    = xtab(1)
      x2    = xtab(2)
      x3    = xtab(3)

      a  = xin - x1
      b  = xin - x2
      c  = xin - x3
      d  = x1 - x2
      e  = x1 - x3
      f  = x2 - x3

      alfa = b*c/(d*e)
      beta = -a*c/(d*f)
      gama = a*b/(e*f)

      yout  = alfa*ytab(1) + beta*ytab(2) + gama*ytab(3)

      return
      end subroutine hardwire_quadratic





      subroutine hardwire_cubic(xtab,ytab,n,xin,yout)
      implicit none

! implements a hardwired cubic interpolation

! declare the pass
      integer  :: n
      real*8   :: xtab(n),ytab(n),xin,yout

! local variables
      real*8   :: x1,x2,x3,x4,a,b,c,d,e,f,g,h,p,q, &
                  alfa,beta,gama,delt

      x1    = xtab(1)
      x2    = xtab(2)
      x3    = xtab(3)
      x4    = xtab(4)

      a  = xin - x1
      b  = xin - x2
      c  = xin - x3
      d  = xin - x4
      e  = x1 - x2
      f  = x1 - x3
      g  = x1 - x4
      h  = x2 - x3
      p  = x2 - x4
      q  = x3 - x4

      alfa =  b*c*d/(e*f*g)
      beta = -a*c*d/(e*h*p)
      gama =  a*b*d/(f*h*q)
      delt = -a*b*c/(g*p*q)

      yout  = alfa*ytab(1) + beta*ytab(2) + gama*ytab(3) + delt*ytab(4)

      return
      end subroutine hardwire_cubic



